#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "config.h"

#define MAXTEXTL 128
#define MAXPATH  512
#define MAXFIT 1024

float * wave, * flux, * bb, * err, *berr;

int aefflag=0,i,nar,ndim;
float dt=1.;
float *aeff, *alam;
/* float *apos, *aneg; */

/*  -------------------------------------------------------------------- */

int read_spec (char *path)
{
  FILE * fd;
  char line[BUFSIZ];
  int count_line = 0, items = 0, count=0;
 
  if (NULL == (fd = fopen(path, "r")))
    {
      fprintf(stderr,
	  "* Warning *: fopen: %s: File not found\n", path);
      return (-1);
    }
 
  while (fgets(line, BUFSIZ-1, fd))
    if (line[0] != '#') ++count;
 
  rewind(fd);
 
  wave = (float *) malloc (sizeof(float) * count);
  flux = (float *) malloc (sizeof(float) * count);
  bb   = (float *) malloc (sizeof(float) * count);
  err  = (float *) malloc (sizeof(float) * count);
  berr = (float *) malloc (sizeof(float) * count);
 
  if (wave == NULL || flux == NULL || bb == NULL || err == NULL)
    {
      fprintf(stderr,"* Warning *: Out of memory\n");
      return(-1);
    }
  while (fgets(line, BUFSIZ-1, fd))
    {
      if (line[0] != '#')
        {
          items = sscanf(line, "%f %f %f %f %f",
                         &wave[count_line], &flux[count_line],
                         &err[count_line], &bb[count_line], &berr[count_line]);
          if (items == 5)
            ++count_line;
          else
            {
              fprintf(stderr, "* Warning * %s: Invalid line: %s", path, line);
              free(wave); free(flux); free(err); free(bb); free(berr);
              return (-1);
            }
        }
 
     }
fclose(fd);
  if (count_line < count)
    {
      fprintf(stderr, "* Warning *: Race condition detected in %s\n",path);
      free(wave); free(flux); free(err); free(bb); free(berr);
      return (-1);
    }
  else
	fprintf(stdout, "Message      : Total: %d datapoints\n", count);
 
  return (count);
}

/*  -------------------------------------------------------------------- */

int get_aeff(char *arfile)
{

FILE *fd;
int items=0,iflag=0;
char line[BUFSIZ];
float al,ar;
 
    if (NULL== (fd=fopen(arfile,"r")))
    {
      printf("* Warning * %s: File not found.\n", arfile);
      return(-1);
    }
    else {
      while (fgets(line, BUFSIZ-1, fd)) if (line[0] != '#') ++nar;
      rewind(fd);}

   aeff = (float *) malloc (sizeof(float) * nar); 
   alam = (float *) malloc (sizeof(float) * nar);
/*   apos = (float *) malloc (sizeof(float) * nar);
     aneg = (float *) malloc (sizeof(float) * nar); */

  if (alam==NULL || aeff==NULL /*|| apos==NULL || aneg==NULL*/)
    {
      fprintf(stderr,"* Warning *: Out of memory\n");
      return(-1);
    }

   i=0;
   while (fgets(line, BUFSIZ-1, fd))
    {
	al = 0.; ar = 0.;
	if (line[0] != '#')
	{items = sscanf(line, "%f %f %f", &alam[i], &al,&ar);
	aeff[i]=al+ar;  /* two columns can be given, to be added */
	if (aeff[i] > al && iflag == 0) {printf("You have given two columns with effective areas. They will be added.\n");++iflag;}
/*	apos[i]=al;
	aneg[i]=ar; */
/*  if (alam[i] < 20.) fprintf(stderr,"%d %f  %f  %f %f\n",i,alam[i],al,ar,aeff[i]); */
	++i;}
    }
return(0);
}

/*  -------------------------------------------------------------------- */ 

void
make_fluxfile (char *path)

{
  FILE *fd;
  int ndigit=0,j=0,vk;
  float test;
  char *fformat;

  fformat= malloc(59);
  test=99999999999999.;
  for (i=1;i<ndim;++i) if (bb[i] <test &&bb[i] != 0.) {test=bb[i];j=i;}
  ndigit = floor((log10(bb[j])));
  if (ndigit<0) {ndigit=-ndigit+2;vk=ndigit+4;}
  else {vk = ndigit+3;ndigit=3;}

  sprintf(fformat,"%s%d.%df  %s%d.%df  %s%d.%df  %s%d.%df  %s%d.%df\n","%",vk,ndigit,"%",vk,ndigit,"%",vk,ndigit,"%",vk,ndigit,"%",vk,ndigit);

  printf("%s",fformat);

  fd = fopen (path, "w");
  if (fd)
        {
          for (i=0;i<ndim;++i)
 /*           fprintf(fd, "%8.3f  %4.2f %4.2f %4.2f %4.2f\n", */
		fprintf(fd, fformat,
                    wave[i], flux[i], err[i], bb[i], berr[i]);
          fclose(fd);
	}

  return;
}

/*  -------------------------------------------------------------------- */

int main(int argc, char *argv[])
{

FILE * fd;
int j=0;
float aval;
char datafile[MAXPATH];
char outfile[MAXPATH];
char usage[MAXPATH];
char arfile[MAXPATH];

strncpy (datafile, "spectrum.dat", MAXPATH);
strncpy (outfile, "flux.dat", MAXPATH);
strncpy (usage, "Usage: cora_flux [-i infile] [-o outfile] [-a arfile] [-e expos. time]\n", MAXPATH);
strncpy (arfile, "cora.aef", MAXPATH);

  while (argc > 1 && argv[1][0] == '-')
    {
      switch (argv[1][1])
        {
        case 'i':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (datafile, argv[1], MAXPATH);
          break;
        case 'a':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (arfile, argv[1], MAXPATH);
	  aefflag = 1;
          break;
        case 'e':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
	  dt = atof(argv[1]);
          break;
        case 'o':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (outfile, argv[1], MAXPATH);
          break;
        default:
          fprintf (stderr, usage);
	  exit(8);
          break;
        }
      ++argv;
      --argc;
    }

  /*  read log file into global variables
   */
    if (NULL== (fd=fopen(datafile,"r"))) 
    {
      printf("* Warning * %s: File not found.\n", datafile);
      return(-1);
    } 
    else fclose(fd);

    ndim = read_spec(datafile);
    assert (ndim !=-1);

    if (aefflag == 1)
     {
       i=get_aeff(arfile);
       for (i=0;i<ndim;++i)
	 {
	   j=1;
	   while (wave[i] > alam[j] && j<nar) ++j;
	   aval = aeff[j-1]+(aeff[j]-aeff[j-1])*(alam[j]-wave[i]);
	   aval *=dt;
	   if (aval != 0.)
	    {
	     flux[i] /= aval;
	     err[i] /= aval;
	     bb[i] /= aval;
	     berr[i] /= aval;
	    }
	   else
	    {
	     flux[i]=0.;
	     err[i]=0.;
	     bb[i]=0.;
	     berr[i]=0.;
	    }
	 }
      }

    make_fluxfile(outfile);

return (0);                                                                     
}
